/* adc.h
 *
 * Created on: Mar 15, 2022
 * Author: Martin Danek, martin@embedblog.eu
 */

#ifndef DAQ_ADC_H_
#define DAQ_ADC_H_

#include <stdint.h>
#include "ADS131A0x.h"

enum class SampleRate: uint8_t
{
	_1_KSPS = 0b00100000,
	_8_KSPS = 0b00100101,
	_32_KSPS = 0b00101011,
	//_64_KSPS = 0b00101101,
	__MAX
};

extern const char* sampleRates[];

namespace adc
{
	void init();
	void drdyCb();
	void dmaCpltCb();

	int32_t getChannelAvgRaw(uint_fast8_t channel);
	uint32_t getChannelRmsRaw(uint_fast8_t channel);

	float getChannelDc(uint_fast8_t channel);
	float getChannelRms(uint_fast8_t channel);
	float getChannelAcRms(uint_fast8_t channel);

	void getChannelMinMax(uint_fast8_t channel, float* min, float* max);
	void resetMinMax();
	void doAutoranging();

	float getAnalogFrequency(uint_fast8_t channel);

	uint8_t readRegisterAsync(ADS131A0x::Reg reg);
	uint8_t readLastStatusAsync();
	void writeRegisterAsync(ADS131A0x::Reg reg, uint8_t value);

	void increaseSampleRate();
	SampleRate getSampleRate();
}



#endif /* DAQ_ADC_H_ */
