/* afe.h
 *
 * Created on: Mar 21, 2022
 * Author: Martin Danek, martin@embedblog.eu
 */

#ifndef DAQ_AFE_H_
#define DAQ_AFE_H_

enum class CommonTerminal
{
	FLOATING,
	REF_2,			//half the reference voltage
	GROUNDED,
	CURRENT_SINK
};

enum class VoltageRange
{
	__MIN = 0,
	_60_mV,
	_600_mV,
	_6_V,
	_60_V,
	_600_V,
	__MAX
};

enum class CurrentRange
{
	__MIN = 0,
	_6_mA,
	_60_mA,
	_600_mA,
	_6_A,
	__MAX
};

enum class AuxResistor
{
	__MIN = 0,
	NONE,
	_510_OHM,
	_1_KILO_OHM,
	_10_KILO_OHM,
	_100_KILO_OHM,
	_1_MEGA_OHM,
	_10_MEGA_OHM,
	__MAX
};

namespace afe
{
	void setCommonTerminal(CommonTerminal commonTerminal);

	void setVoltageRange(VoltageRange voltageRange);
	VoltageRange getVoltageRange();
	void increaseVoltageRange(bool isCircularAllowed);
	void decreaseVoltageRange(bool isCircularAllowed);

	void setCurrentRange(CurrentRange currentRange);
	CurrentRange getCurrentRange();
	void increaseCurrentRange(bool isCircularAllowed);
	void decreaseCurrentRange(bool isCircularAllowed);

	void setAuxResistor(AuxResistor auxResistor);
	AuxResistor getAuxResistor();
	void decreaseAuxResistor();
}



#endif /* DAQ_AFE_H_ */
