/* thermocouple.cpp
 *
 * Created on: 6. 5. 2022
 * Author: Martin Danek, martin@embedblog.eu
 * Source: https://srdata.nist.gov/its90/type_b/bcoefficients_inverse.html
 */

#include "thermocouple.h"

#include <stddef.h>
#include <math.h>

constexpr static size_t K_NUM_COEFF = 10;
constexpr static float K_COEFF[][K_NUM_COEFF] =
{
	{0.000000E0, 	2.5173462E1, 	-1.1662878E0, 	-1.0833638E0, 	-8.9773540E-1, 	-3.7342377E-1, 	-8.6632643E-2, 	-1.0450598E-2, 	-5.1920577E-4,	0.000000E0},
	{0.000000E0, 	2.508355E1, 	7.860106E-2, 	-2.503131E-1, 	8.315270E-2, 	-1.228034E-2, 	9.804036E-4, 	-4.413030E-5, 	1.057734E-6,	-1.052755E-8},
	{-1.318058E2,	4.830222E1,		-1.646031E0,	5.464731E-2,	-9.650715E-4,	8.802193E-6, 	-3.110810E-8,	0.000000E0,		0.000000E0,		0.000000E0}
};


float thermocouple::getTemperature_Ktype(float voltage_mv)
{
	size_t index;
	if ((voltage_mv >= -5.891f) && (voltage_mv < 0.0f))
		index = 0;
	else if ((voltage_mv >= 0.0f) && (voltage_mv < 20.644f))
		index = 1;
	else if ((voltage_mv >= 20.644f) && (voltage_mv <= 54.886f))
		index = 2;
	else
		return 0.0f;

	float retval = K_COEFF[index][0];
	for (size_t j = 1; j < K_NUM_COEFF; j++)
		retval += K_COEFF[index][j] * powf(voltage_mv, j);

	return retval;
}


