/* display.cpp
 *
 * Created on: 15. 3. 2022
 * Author: Martin Danek, martin@embedblog.eu
 */

#include "display.h"

#include "screen_include.h"
#include "main.h"
#include "config.h"
#include "sys.h"

extern SPI_HandleTypeDef hspi3;
UC1608_240x120 lcd{&hspi3, nullptr, 0, LCD_DC_GPIO_Port, LCD_DC_Pin, nullptr, 0};

const Screen* currentScreen;
static uint32_t screenChangeTick;
static bool isHoldEnabled;
static bool isRelEnabled;
static bool isRelSampled;
static float relValue;

void display::init()
{
	MX_SPI3_Init();

	lcd.init(25);
	currentScreen = &screen_init;
	currentScreen->onEntry();

	HAL_Delay(INIT_SCREEN_DELAY_MS);

	changeScreen(&screen_volt);
}

void display::update()
{
	if (HAL_GetTick() - screenChangeTick < SCREEN_CHANGE_TIME)
		return;

	if (isHoldEnabled)
		return;

	currentScreen->onUpdate();
}

void display::action(const ScreenAction screenAction)
{
	if (screenAction == ScreenAction::BTN5_PRESS)
		sys::powerDown();
	else if (screenAction == ScreenAction::BTN6_PRESS)
		isHoldEnabled = !isHoldEnabled;
	else if (screenAction == ScreenAction::BTN7_PRESS)
	{
		isRelEnabled = !isRelEnabled;

		if (!isRelEnabled)
			isRelSampled = false;
	}
	else if (screenAction == ScreenAction::BTN8_PRESS)
		HAL_GPIO_TogglePin(LCD_LED_GPIO_Port, LCD_LED_Pin);
	else
	{
		currentScreen->onAction(screenAction);
		update();
	}

}


bool checkRelMode(float* value)
{
	if (!isRelEnabled)
		return false;

	if (isRelSampled)
		*value -= relValue;
	else
	{
		relValue = *value;
		isRelSampled = true;
	}

	return true;
}

void doNothing()
{
}


void doNothing(const ScreenAction action)
{
	(void)action;
}


void changeScreen(const Screen* const newScreen)
{
	currentScreen->onExit();
	currentScreen = newScreen;
	screenChangeTick = HAL_GetTick();
	currentScreen->onEntry();
}
