/* screen_cap.cpp
 *
 * Created on: 17. 4. 2022
 * Author: Martin Danek, martin@embedblog.eu
 */

#include "screen_include.h"
#include "SansMono_60p.h"
#include "SansMono_24p.h"
#include "SansMono_24p_Bold.h"
#include "SansMono_16p.h"
#include "SansMono_16p_Bold.h"

#include "adc.h"
#include "afe.h"
#include "config.h"
#include "display.h"
#include "main.h"

extern uint32_t refResistors[];
const char* fRangeNames[] = {"200uF", "100uF", "10uF", "1uF", "100nF", "10nF"};
const char* fUnits[] = {"u", "u", "u", "u", "n", "n"};

extern TIM_HandleTypeDef htim2;
extern COMP_HandleTypeDef hcomp3;

//ideal measurement time: 100 ms
//at 1kOhm: 100uF
//at 10kOhm: 10uF
//at 100kOhm: 1uF
//at 1MOhm: 100nF
//at 10MOhm: 10 nF
static void onUpdate()
{
	lcd.clear();
	drawSystemBar();

	HAL_GPIO_WritePin(DISCH_GPIO_Port, DISCH_Pin, GPIO_PIN_RESET);

	auto tickStart = HAL_GetTick();
	auto timerStart = __HAL_TIM_GET_COUNTER(&htim2);
	bool timeout = false;

	while (HAL_COMP_GetOutputLevel(&hcomp3) == COMP_OUTPUT_LEVEL_LOW)		//TODO: remove timeout
		if (HAL_GetTick() - tickStart > CAP_MEAS_TIMEOUT)
		{
			timeout = true;
			break;
		}

	uint32_t tau = __HAL_TIM_GET_COUNTER(&htim2) - timerStart;
	size_t rangeUsed = (size_t)afe::getAuxResistor() - 2;

	float capacity;
	if (timeout)
		capacity = 0;
	else
		capacity = (float)tau / ((refResistors[rangeUsed] + AUX_PROTECTION_RESISTANCE) * 10);

	lcd.setFont(&SansMono_60p);
	lcd.printf(120, 36, Align::CENTER, "%.3f %s", capacity, fUnits[rangeUsed]);

	lcd.drawRectFilled(0, 104, 240, 120 - 104, COLOR_BLACK);
	lcd.setPrimaryColor(COLOR_WHITE);
	lcd.setBgColor(COLOR_BLACK);
	lcd.setFont(&SansMono_16p);
	lcd.print(20, 118, Align::CENTER, "5/6");
	lcd.print(150, 118, Align::CENTER, fRangeNames[rangeUsed]);
	lcd.setPrimaryColor(COLOR_BLACK);
	lcd.setBgColor(COLOR_WHITE);

	lcd.displayDma();

	afe::setAuxResistor(AuxResistor::NONE);
	HAL_GPIO_WritePin(DISCH_GPIO_Port, DISCH_Pin, GPIO_PIN_SET);
}


static void onEntry()
{
	lcd.clear();
	lcd.setFont(&SansMono_24p_Bold);
	lcd.printf(120, 90, Align::CENTER, "CAPACITY MODE");
	lcd.displayDma();

	afe::setCommonTerminal(CommonTerminal::GROUNDED);
	afe::setAuxResistor(AuxResistor::_1_KILO_OHM);
	HAL_GPIO_WritePin(DISCH_GPIO_Port, DISCH_Pin, GPIO_PIN_SET);
	HAL_Delay(SCREEN_CHANGE_TIME);
}


static void onExit()
{
	afe::setCommonTerminal(CommonTerminal::REF_2);
}


static void onAction(const ScreenAction action)
{
	switch (action)
	{
		case ScreenAction::BTN1_PRESS:
			changeScreen(&screen_thermo);
			break;
		case ScreenAction::BTN3_PRESS:
			afe::decreaseAuxResistor();
			break;
		default:
			break;
	}
}

const Screen screen_cap =
{
	.onUpdate = onUpdate,
	.onEntry = onEntry,
	.onExit = onExit,
	.onAction = onAction
};
