/* screen_power.cpp
 *
 * Created on: May 2, 2022
 * Author: Martin Danek, martin@embedblog.eu
 */

#include "screen_include.h"
#include "SansMono_60p.h"
#include "SansMono_24p.h"
#include "SansMono_24p_Bold.h"
#include "SansMono_16p.h"
#include "SansMono_16p_Bold.h"

#include "adc.h"
#include "afe.h"
#include "config.h"
#include "display.h"

#include "custom_log.h"

extern const char* vRangeNames[], iRangeNames[];
extern const char* vUnits[], iUnits[];
const char* pPrintFormats[] = {"%5.1f", "%5.3f", "%5.2f", "%5.1f", "%5.3f", "%5.2f", "%5.1f", "%5.3f"};
const char* pUnits[] = {"u", "m", "m", "m", "", "", "", "k"};

static uint_fast8_t currentView;
static constexpr uint8_t MAX_VIEW = 3;


static void onUpdate()
{
	lcd.clear();
	drawSystemBar();
	size_t vRange = (size_t)afe::getVoltageRange() - 1;
	size_t iRange = (size_t)afe::getCurrentRange() - 1;
	size_t pRange =  vRange + iRange;

	float vDc = adc::getChannelDc(ADC_VOLT_CH);
	float vRms = adc::getChannelRms(ADC_VOLT_CH);
	float iDc = adc::getChannelDc(ADC_CURR_CH);
	float iRms = adc::getChannelRms(ADC_CURR_CH);
	float realPower = vDc * iDc;
	float apparentPower = vRms * iRms;

	switch (currentView)
	{
		case 0:
		{
			lcd.setFont(&SansMono_60p);
			lcd.printf(120, 61, Align::CENTER, pPrintFormats[pRange], realPower);

			lcd.setFont(&SansMono_24p_Bold);
			lcd.printf(240, 83, Align::RIGHT, "%sW", pUnits[pRange]);

			lcd.setFont(&SansMono_16p);
			lcd.drawHorLine(0, 84, 240, COLOR_BLACK);
			lcd.printf(0, 100, "%s %s", vRangeNames[vRange], iRangeNames[iRange]);
			break;
		}
		case 1:
		{
			lcd.setFont(&SansMono_60p);
			lcd.printf(120, 61, Align::CENTER, pPrintFormats[pRange], apparentPower);

			lcd.setFont(&SansMono_24p_Bold);
			lcd.printf(240, 83, Align::RIGHT, "%sVA", pUnits[pRange]);

			lcd.setFont(&SansMono_16p);
			lcd.drawHorLine(0, 84, 240, COLOR_BLACK);
			lcd.printf(0, 100, "%s %s", vRangeNames[vRange], iRangeNames[iRange]);
			break;
		}
		case 2:
		{
			lcd.setFont(&SansMono_24p_Bold);
			lcd.printf(120, 36, Align::CENTER, "%.3f %s", vDc, vUnits[vRange]);
			lcd.printf(120, 61, Align::CENTER, "%.3f %s", iDc, iUnits[iRange]);
			lcd.printf(120, 84, Align::CENTER, "%.3f %s", realPower, pUnits[pRange]);

			lcd.setFont(&SansMono_16p);
			lcd.drawHorLine(0, 84, 240, COLOR_BLACK);
			lcd.printf(0, 100, "F: %u Hz", adc::getAnalogFrequency(ADC_VOLT_CH));
			break;
		}
	}

	lcd.drawRectFilled(0, 104, 240, 120 - 104, COLOR_BLACK);
	lcd.setPrimaryColor(COLOR_WHITE);
	lcd.setBgColor(COLOR_BLACK);
	lcd.setFont(&SansMono_16p);
	lcd.print(20, 118, Align::CENTER, "3/6");
	lcd.printf(90, 118, Align::CENTER, "%u/3", currentView + 1);
	lcd.print(150, 118, Align::CENTER, "AUTO");
	lcd.print(240, 118, Align::RIGHT, sampleRates[(size_t)adc::getSampleRate()]);
	lcd.setPrimaryColor(COLOR_BLACK);
	lcd.setBgColor(COLOR_WHITE);

	lcd.displayDma();
	adc::doAutoranging();
}


static void onEntry()
{
	lcd.clear();
	lcd.setFont(&SansMono_24p_Bold);
	lcd.printf(120, 90, Align::CENTER, "POWER MODE");
	lcd.displayDma();

	HAL_Delay(SCREEN_CHANGE_TIME);
}


static void onAction(const ScreenAction action)
{
	switch (action)
	{
		case ScreenAction::BTN1_PRESS:
			changeScreen(&screen_res);
			break;
		case ScreenAction::BTN2_PRESS:
			currentView++;
			if (currentView >= MAX_VIEW)
				currentView = 0;
			break;
		case ScreenAction::BTN4_PRESS:
			adc::increaseSampleRate();
			break;
		default:
			break;
	}
}

const Screen screen_power =
{
	.onUpdate = onUpdate,
	.onEntry = onEntry,
	.onAction = onAction
};
