/* screen_res.cpp
 *
 * Created on: April 23, 2022
 * Author: Martin Danek, martin@embedblog.eu
 */

#include "screen_include.h"
#include "SansMono_60p.h"
#include "SansMono_24p.h"
#include "SansMono_24p_Bold.h"
#include "SansMono_16p.h"
#include "SansMono_16p_Bold.h"

#include "adc.h"
#include "afe.h"
#include "config.h"
#include "display.h"
#include "main.h"

uint32_t refResistors[] = {510, 1000, 10000, 100000, 1000000, 10000000};
const char* rRangeNames[] = {"500Ohm", "1kOhm", "10kOhm", "100kOhm", "1MOhm", "10MOhm"};
const char* rUnits[] = {"Ohm", "kOhm", "kOhm", "kOhm", "MOhm", "MOhm"};

static bool isBeepEnabled;

static void onUpdate()
{
	lcd.clear();
	drawSystemBar();

	auto adc_raw = adc::getChannelAvgRaw(ADC_AUX_CH);
	size_t rangeUsed = (size_t)afe::getAuxResistor() - 2;
	float res = refResistors[rangeUsed] / ((float)INT24_MAX / adc_raw - 1) - AUX_PROTECTION_RESISTANCE;

	lcd.setFont(&SansMono_60p);
	lcd.printf(120, 36, Align::CENTER, "%.3f %s", res, rUnits[rangeUsed]);

	if (res < AUX_BEEP_THRESHOLD)
		HAL_GPIO_WritePin(BUZZ_GPIO_Port, BUZZ_Pin, GPIO_PIN_SET);
	else
		HAL_GPIO_WritePin(BUZZ_GPIO_Port, BUZZ_Pin, GPIO_PIN_RESET);

	lcd.drawRectFilled(0, 104, 240, 120 - 104, COLOR_BLACK);
	lcd.setPrimaryColor(COLOR_WHITE);
	lcd.setBgColor(COLOR_BLACK);
	lcd.setFont(&SansMono_16p);
	lcd.print(20, 118, Align::CENTER, "4/6");
	if (isBeepEnabled)
		lcd.print(90, 118, Align::CENTER, "CONT");
	lcd.print(150, 118, Align::CENTER, rRangeNames[rangeUsed]);
	lcd.setPrimaryColor(COLOR_BLACK);
	lcd.setBgColor(COLOR_WHITE);

	lcd.displayDma();
}


static void onEntry()
{
	lcd.clear();
	lcd.setFont(&SansMono_24p_Bold);
	lcd.printf(120, 90, Align::CENTER, "RESISTANCE MODE");
	lcd.displayDma();
	afe::setAuxResistor(AuxResistor::_10_MEGA_OHM);
	afe::setCommonTerminal(CommonTerminal::GROUNDED);

	HAL_Delay(SCREEN_CHANGE_TIME);
}


static void onExit()
{
	HAL_GPIO_WritePin(BUZZ_GPIO_Port, BUZZ_Pin, GPIO_PIN_RESET);
	afe::setCommonTerminal(CommonTerminal::REF_2);
}


static void onAction(const ScreenAction action)
{
	switch (action)
	{
		case ScreenAction::BTN1_PRESS:
			changeScreen(&screen_cap);
			break;
		case ScreenAction::BTN2_PRESS:
			isBeepEnabled = !isBeepEnabled;
			break;
		case ScreenAction::BTN3_PRESS:
			afe::decreaseAuxResistor();
			break;
		default:
			break;
	}
}


const Screen screen_res =
{
	.onUpdate = onUpdate,
	.onEntry = onEntry,
	.onExit = onExit,
	.onAction = onAction
};
