/* screen_thermo.cpp
 *
 * Created on: 17. 4. 2022
 * Author: Martin Danek, martin@embedblog.eu
 */

#include "screen_include.h"
#include "SansMono_60p.h"
#include "SansMono_16p.h"
#include "SansMono_24p_Bold.h"

#include "adc.h"
#include "config.h"
#include "sys.h"
#include "thermocouple.h"

static void onUpdate()
{
	lcd.clear();
	drawSystemBar();

	lcd.setFont(&SansMono_16p);
	auto voltage_mV = adc::getChannelDc(ADC_THERM_CH) * 1000.0f;
	auto tc_temp = thermocouple::getTemperature_Ktype(voltage_mV);
	auto int_temp = sys::getTemperature();
	auto result = int_temp + tc_temp;

	lcd.setFont(&SansMono_60p);
	lcd.printf(120, 36, Align::CENTER, "%.1f", result);

	lcd.setFont(&SansMono_24p_Bold);
	lcd.print(240, 83, Align::RIGHT, "oC");

	lcd.drawRectFilled(0, 104, 240, 120 - 104, COLOR_BLACK);
	lcd.setPrimaryColor(COLOR_WHITE);
	lcd.setBgColor(COLOR_BLACK);
	lcd.setFont(&SansMono_16p);
	lcd.print(20, 118, Align::CENTER, "6/6");
	lcd.setPrimaryColor(COLOR_BLACK);
	lcd.setBgColor(COLOR_WHITE);

	lcd.displayDma();
}

static void onEntry()
{
	lcd.clear();
	lcd.setFont(&SansMono_24p_Bold);
	lcd.printf(120, 90, Align::CENTER, "THERMOCOUPLE");
	lcd.displayDma();

	HAL_Delay(SCREEN_CHANGE_TIME);
}

static void onAction(const ScreenAction action)
{
	switch (action)
	{
		case ScreenAction::BTN1_PRESS:
			changeScreen(&screen_volt);
			break;
		default:
			break;
	}
}

const Screen screen_thermo =
{
	.onUpdate = onUpdate,
	.onEntry = onEntry,
	.onAction = onAction
};
