/* config.h
 *
 * Created on: 15. 3. 2022
 * Author: Martin Danek, martin@embedblog.eu
 */

#ifndef CONFIG_H_
#define CONFIG_H_

#include <stdint.h>

constexpr auto INIT_SCREEN_DELAY_MS = 1500;

constexpr uint32_t SCREEN_REFRESH_MS = 500;

constexpr float VREF = 2.442f;
constexpr float ADC_INTERNAL_LSB_SIZE = VREF / 4096;
constexpr float ADS131_LSB_SIZE = (2 * VREF) / 16777216;

constexpr uint8_t IOEXP_BTN_1_PIN = 0;
constexpr uint8_t IOEXP_BTN_2_PIN = 1;
constexpr uint8_t IOEXP_BTN_3_PIN = 2;
constexpr uint8_t IOEXP_BTN_4_PIN = 3;
constexpr uint8_t IOEXP_BTN_5_PIN = 4;
constexpr uint8_t IOEXP_BTN_6_PIN = 5;
constexpr uint8_t IOEXP_BTN_7_PIN = 6;
constexpr uint8_t IOEXP_BTN_8_PIN = 7;

constexpr uint8_t IOEXP_BTN_1_MASK = (1 << IOEXP_BTN_1_PIN);
constexpr uint8_t IOEXP_BTN_2_MASK = (1 << IOEXP_BTN_2_PIN);
constexpr uint8_t IOEXP_BTN_3_MASK = (1 << IOEXP_BTN_3_PIN);
constexpr uint8_t IOEXP_BTN_4_MASK = (1 << IOEXP_BTN_4_PIN);
constexpr uint8_t IOEXP_BTN_5_MASK = (1 << IOEXP_BTN_5_PIN);
constexpr uint8_t IOEXP_BTN_6_MASK = (1 << IOEXP_BTN_6_PIN);
constexpr uint8_t IOEXP_BTN_7_MASK = (1 << IOEXP_BTN_7_PIN);
constexpr uint8_t IOEXP_BTN_8_MASK = (1 << IOEXP_BTN_8_PIN);

constexpr size_t VOLTAGE_RANGES = 5;
constexpr size_t CURRENT_RANGES = 4;
constexpr uint_fast8_t ADC_VOLT_CH = 0;
constexpr uint_fast8_t ADC_CURR_CH = 1;
constexpr uint_fast8_t ADC_AUX_CH = 2;
constexpr uint_fast8_t ADC_THERM_CH = 3;

constexpr uint32_t SCREEN_CHANGE_TIME = 1000;		//how long does the screen change text show up

constexpr float AUX_PROTECTION_RESISTANCE = 60.0f;
constexpr float AUX_BEEP_THRESHOLD = 50.0f;
constexpr uint32_t CAP_MEAS_TIMEOUT = 120;

constexpr int32_t INT24_MAX = 8388607;
constexpr int32_t INT24_MIN = -8388608;
constexpr int32_t AUTORANGE_MARGIN = 830000;						//how close to end-of-range/middle we need to be to switch ranges
constexpr int32_t AUTORANGE_UP = INT24_MAX - AUTORANGE_MARGIN;		//below this value, we switch ranges
constexpr int32_t AUTORANGE_DOWN = AUTORANGE_MARGIN;				//above this value, we switch ranges



#define ATTR_FAST 				__attribute__((optimize("Ofast")))

#endif /* CONFIG_H_ */
