#ifndef GFX_CORE_H_
#define GFX_CORE_H_

#include <stdint.h>

#ifndef GFX_PRINTF_BUFFER
#define GFX_PRINTF_BUFFER		64				//TODO: param
#endif

typedef struct
{
  uint16_t bitmapOffset; 		// Pointer into GFXfont->bitmap
  uint8_t width;         		// Bitmap dimensions in pixels
  uint8_t height;        		// Bitmap dimensions in pixels
  uint8_t xAdvance;      		// Distance to advance cursor (x axis)
  int8_t xOffset;        		// X dist from cursor pos to UL corner
  int8_t yOffset;        		// Y dist from cursor pos to UL corner
} GFXglyph;

// Data stored for FONT AS A WHOLE
typedef struct
{
  uint8_t *bitmap;  			// Glyph bitmaps, concatenated
  GFXglyph *glyph;  			// Glyph array
  uint16_t first;   			// ASCII extents (first char)
  uint16_t last;    			// ASCII extents (last char)
  uint8_t yAdvance; 			// Newline distance (y axis)
} GFXfont;

enum class Align
{
	LEFT,
	CENTER,
	RIGHT
};


//https://github.com/adafruit/Adafruit-GFX-Library
class GFX_core
{
public:
	GFX_core(const uint16_t w, const uint16_t h, void* const buf, const uint32_t bufPosMax):
		m_cursor_x(0), m_cursor_y(0), m_rotation(0), m_primColor(0),
		m_bgColor(0), m_width(w), m_height(h), m_dispBuffer(buf), m_bufPosMax(bufPosMax) {};

	virtual void setRotation(uint8_t r);
	void setCursor(uint16_t x, uint16_t y);
	void setPrimaryColor(uint16_t color);
	void setBgColor(uint16_t color);

	void clear();
	virtual void fillScreen(uint16_t color) = 0;

	virtual bool drawPixel(uint16_t x, uint16_t y, uint16_t color) = 0;

	void drawHorLine(uint16_t x, uint16_t y, uint16_t w, uint16_t color);
	void drawVertLine(uint16_t x, uint16_t y, uint16_t h, uint16_t color);
	void drawLine(int16_t x0, int16_t y0, int16_t x1, int16_t y1, uint16_t color);

	void drawRect(uint16_t x, uint16_t y, uint16_t w, uint16_t h, uint16_t color);
	void drawRectFilled(uint16_t x, uint16_t y, uint16_t w, uint16_t h, uint16_t color);

	void drawCircle(uint16_t center_x, uint16_t center_y, uint16_t radius, uint16_t color);
	void drawCircleFilled(uint16_t center_x, uint16_t center_y, uint16_t radius, uint16_t color);
	void drawCircleHelper(uint16_t center_x, uint16_t center_y, uint16_t radius, uint8_t corners, int16_t delta, uint16_t color);

	void setFont(const GFXfont *f);

	void drawChar(int16_t x, int16_t y, unsigned char c, uint16_t color);

	void print(const char* str);
	void print(int16_t x, int16_t y, const char* str);
	void print(int16_t x, int16_t y, Align a_align, const char* str);

	void printf(const char *szFormat, ...);
	void printf(int16_t x, int16_t y, const char *szFormat, ...);
	void printf(int16_t x, int16_t y, Align a_align, const char *szFormat, ...);

	void printVertical(int16_t xLeft, int16_t yTop, uint8_t yAdvance, const char* str);

	uint16_t getStringWidth(const char* str);



protected:
	template <typename T>
	void swap(T* a, T* b) {T temp = *a; *a = *b; *b = temp;};

	uint16_t m_cursor_x;
	uint16_t m_cursor_y;
	uint8_t m_rotation;

	uint16_t m_primColor;
	uint16_t m_bgColor;

	uint16_t m_width;
	uint16_t m_height;

	const GFXfont* m_font;

	void* m_dispBuffer;
	const uint32_t m_bufPosMax;
};



#endif /* GFX_CORE_H_ */
