#ifndef UC1608_H_
#define UC1608_H_

#include "hal_inc.h"
#include "GFX_mono.h"

#ifndef UC1608_SPI_TIMEOUT
#define UC1608_SPI_TIMEOUT 100
#endif

class UC1608: public GFX_mono
{
public:
	UC1608(SPI_HandleTypeDef* spi, GPIO_TypeDef* cs_port, uint16_t cs_pin, GPIO_TypeDef* dc_port, uint16_t dc_pin,
			GPIO_TypeDef* rst_port, uint16_t rst_pin, uint16_t width, uint16_t height, uint8_t* displayBuffer, uint32_t displaySize);

	void init(uint8_t potentiometer = 15);
	void reset(void);

	void setRotation(uint8_t r);
	void setVlcd(uint8_t gain, uint8_t pm);

	void enable(bool enabled);
	void invert(bool inverted);
	void allPixelsOn(bool allOn);

	void display(void);

	void displayDma(void);
	bool isDmaInProgress(void);

private:
	SPI_HandleTypeDef* const m_spi;

	GPIO_TypeDef* const m_cs_port;
	const uint16_t m_cs_pin;
	GPIO_TypeDef* const m_dc_port;
	const uint16_t m_dc_pin;
	GPIO_TypeDef* const m_rst_port;
	const uint16_t m_rst_pin;

	void sendCommand(uint8_t command);
	void sendCommand(uint8_t* p_cmds, uint8_t num);
};


class UC1608_240x120: public UC1608
{
public:
	UC1608_240x120(	SPI_HandleTypeDef* spi, GPIO_TypeDef* cs_port, uint16_t cs_pin, GPIO_TypeDef* dc_port, uint16_t dc_pin,
					GPIO_TypeDef* rst_port, uint16_t rst_pin):
						UC1608(spi, cs_port, cs_pin, dc_port, dc_pin, rst_port, rst_pin, width, height, m_buffer, bufSize)
	{};

private:
	static constexpr uint16_t width = 240;
	static constexpr uint16_t height = 120;
	static constexpr uint32_t bufSize = width * height / 8;

	uint8_t m_buffer[bufSize];
};


#endif /* UC1608_H_ */
