#include "GFX_core.h"

#include <math.h>
#include <stdio.h>
#include <stdarg.h>


void GFX_core::setRotation(uint8_t r)
{
	m_rotation = r % 4;
}


void GFX_core::setCursor(uint16_t x, uint16_t y)		//TODO: maybe a w/h check?
{
	m_cursor_x = x;
	m_cursor_y = y;
}


void GFX_core::setPrimaryColor(uint16_t color)
{
	m_primColor = color;
}


void GFX_core::setBgColor(uint16_t color)
{
	m_bgColor = color;
}


void GFX_core::clear()
{
	fillScreen(m_bgColor);
}


void GFX_core::drawHorLine(uint16_t x, uint16_t y, uint16_t w, uint16_t color)
{
	for (uint16_t xpos = x; xpos < (x + w); xpos++) drawPixel(xpos, y, color);
}


void GFX_core::drawVertLine(uint16_t x, uint16_t y, uint16_t h, uint16_t color)
{
	for (uint16_t ypos = y; ypos < (y + h); ypos++) drawPixel(x, ypos, color);
}


void GFX_core::drawLine(int16_t x0, int16_t y0, int16_t x1, int16_t y1, uint16_t color)
{
	int16_t steep = abs(y1 - y0) > abs(x1 - x0);				//TODO: uint16_t as arguments?
	if (steep)
	{
		swap<int16_t>(&x0, &y0);
		swap<int16_t>(&x1, &y1);
	}

	if (x0 > x1)
	{
		swap<int16_t>(&x0, &x1);
		swap<int16_t>(&y0, &y1);
	}

	int16_t dx, dy;
	dx = x1 - x0;
	dy = abs(y1 - y0);

	int16_t err = dx / 2;
	int16_t ystep;

	if (y0 < y1)
		ystep = 1;
  	else
  		ystep = -1;

	for (; x0 <= x1; x0++)
	{
		if (steep)
			drawPixel(y0, x0, color);
		else
			drawPixel(x0, y0, color);

		err -= dy;
		if (err < 0)
		{
			y0 += ystep;
			err += dx;
		}
	}
}


void GFX_core::drawRect(uint16_t x, uint16_t y, uint16_t w, uint16_t h, uint16_t color)
{
	drawVertLine(x, y, h, color);
	drawVertLine(x + w, y, h, color);
	drawHorLine(x, y, w, color);
	drawHorLine(x, y + h, w, color);
}


void GFX_core::drawRectFilled(uint16_t x, uint16_t y, uint16_t w, uint16_t h, uint16_t color)
{
	for (uint16_t xCurrent = x; xCurrent < (x + w); xCurrent++)
		drawVertLine(xCurrent, y, h, color);
}


void GFX_core::drawCircle(uint16_t center_x, uint16_t center_y, uint16_t radius, uint16_t color)
{
	int16_t f = 1 - radius;
	int16_t ddF_x = 1;
	int16_t ddF_y = -2 * radius;
	int16_t x = 0;
	int16_t y = radius;

	drawPixel(center_x, center_y + radius, color);
	drawPixel(center_x, center_y - radius, color);
	drawPixel(center_x + radius, center_y, color);
	drawPixel(center_x - radius, center_y, color);

	while (x < y)
	{
		if (f >= 0)
		{
			y--;
			ddF_y += 2;
			f += ddF_y;
		}

		x++;
		ddF_x += 2;
		f += ddF_x;

		drawPixel(center_x + x, center_y + y, color);
		drawPixel(center_x - x, center_y + y, color);
		drawPixel(center_x + x, center_y - y, color);
		drawPixel(center_x - x, center_y - y, color);
		drawPixel(center_x + y, center_y + x, color);
		drawPixel(center_x - y, center_y + x, color);
		drawPixel(center_x + y, center_y - x, color);
		drawPixel(center_x - y, center_y - x, color);
  }
}


void GFX_core::drawCircleFilled(uint16_t center_x, uint16_t center_y, uint16_t radius, uint16_t color)
{
	drawVertLine(center_x, center_y - radius, 2 * radius + 1, color);
	drawCircleHelper(center_x, center_y, radius, 3, 0, color);
}


void GFX_core::drawCircleHelper(uint16_t center_x, uint16_t center_y, uint16_t radius, uint8_t corners, int16_t delta, uint16_t color)
{

	int16_t f = 1 - radius;
	int16_t ddF_x = 1;
	int16_t ddF_y = -2 * radius;
	int16_t x = 0;
	int16_t y = radius;
	int16_t px = x;
	int16_t py = y;

	delta++; // Avoid some +1's in the loop

	while (x < y)
	{
		if (f >= 0)
		{
			y--;
			ddF_y += 2;
			f += ddF_y;
		}

		x++;
		ddF_x += 2;
		f += ddF_x;

		// These checks avoid double-drawing certain lines, important
		// for the SSD1306 library which has an INVERT drawing mode.
		if (x < (y + 1))
		{
			if (corners & 1)
				drawVertLine(center_x + x, center_y - y, 2 * y + delta, color);
			if (corners & 2)
				drawVertLine(center_x - x, center_y - y, 2 * y + delta, color);
		}
		if (y != py)
		{
			if (corners & 1)
				drawVertLine(center_x + py, center_y - px, 2 * px + delta, color);
			if (corners & 2)
				drawVertLine(center_x - py, center_y - px, 2 * px + delta, color);
			py = y;
		}
		px = x;
	}
}


void GFX_core::setFont(const GFXfont *f)
{
	m_font = (GFXfont*)f;
}


void GFX_core::drawChar(int16_t x, int16_t y, unsigned char c, uint16_t color)
{
	c -= m_font->first;
	GFXglyph *glyph  = &((m_font->glyph)[c]);
	uint8_t  *bitmap = m_font->bitmap;

	uint16_t bo = glyph->bitmapOffset;
	uint8_t w = glyph->width, h = glyph->height;
	int8_t xo = glyph->xOffset, yo = glyph->yOffset;
	uint8_t xx, yy, bits = 0, bit = 0;

	for (yy = 0; yy < h; yy ++)
	{
		for (xx = 0; xx < w; xx ++)
		{
			if (!(bit++ & 7)) bits = bitmap[bo++];
			if (bits & 0x80) drawPixel(x + xo + xx, y + yo + yy, color);
			bits <<= 1;
		}
	}

	m_cursor_x += glyph->xAdvance;
}


void GFX_core::print(const char* str)
{
	while (*str) drawChar(m_cursor_x, m_cursor_y, *str++, m_primColor);
}


void GFX_core::print(int16_t x, int16_t y, const char* str)
{
	m_cursor_x = x;
	m_cursor_y = y;
	print(str);
}


void GFX_core::print(int16_t x, int16_t y, Align a_align, const char* str)
{
	uint16_t width = getStringWidth(str);

	switch (a_align)
	{
		case Align::LEFT: print(x, y, str); break;
		case Align::CENTER: print(x - width/2, y, str); break;
		case Align::RIGHT: print(x - width, y, str); break;
	}
}


void GFX_core::printf(const char *szFormat, ...)
{
	char szBuffer[GFX_PRINTF_BUFFER];
	va_list pArgs;
	va_start(pArgs, szFormat);
	vsnprintf(szBuffer, GFX_PRINTF_BUFFER, szFormat, pArgs);
	va_end(pArgs);

	print(szBuffer);
}


void GFX_core::printf(int16_t x, int16_t y, const char* szFormat, ...)
{
	char szBuffer[GFX_PRINTF_BUFFER];
	va_list pArgs;
	va_start(pArgs, szFormat);
	vsnprintf(szBuffer, GFX_PRINTF_BUFFER, szFormat, pArgs);
	va_end(pArgs);

	print(x, y, szBuffer);
}


void GFX_core::printf(int16_t x, int16_t y, Align a_align, const char *szFormat, ...)
{
	char szBuffer[GFX_PRINTF_BUFFER];
	va_list pArgs;
	va_start(pArgs, szFormat);
	vsnprintf(szBuffer, GFX_PRINTF_BUFFER, szFormat, pArgs);
	va_end(pArgs);

	print(x, y, a_align, szBuffer);
}


void GFX_core::printVertical(int16_t xLeft, int16_t yTop, uint8_t yAdvance, const char* str)
{
	//yTop += m_font->yAdvance;
	yTop += yAdvance;

	while (*str)
	{
		char c = *str++;
		drawChar(xLeft, yTop, c, m_primColor);
		yTop += yAdvance;
	}
}


uint16_t GFX_core::getStringWidth(const char* str)
{
	uint16_t width = 0;

	while (*str)
	{
		uint8_t c = (*str++) - m_font->first;
		GFXglyph* glyph  = &((m_font->glyph)[c]);
		width += glyph->xAdvance;
	}

	return width;
}
