/* custom_log.h
 *
 * Created on: 16. 3. 2022
 * Author: Martin Danek, martin@embedblog.eu
 */

#ifndef CUSTOM_LOG_H_
#define CUSTOM_LOG_H_

#include <stddef.h>

enum class DebugLevel
{
	VERBOSE = 0,
	DBG = 1,
	INFO = 2,
	WARNING = 3,
	ERROR = 4
};

#ifndef LOG_LEVEL
#define LOG_LEVEL		0		//sets the minimal level of log, which will be outputted
#endif

void customLog(DebugLevel debugLevel, const char* msg, ...);

//these two functions must be iomplemented by the user:
void _log_send(const char* msg, size_t len);
bool _log_isReady();

#if (LOG_LEVEL < 1)
#define LOGV(...)				customLog(DebugLevel::VERBOSE, __VA_ARGS__)
#else
#define LOGV(...)				(void)(...)
#endif

#if (LOG_LEVEL < 2)
#define LOGD(...)				customLog(DebugLevel::DBG, __VA_ARGS__)
#else
#define LOGD(...)				(void)(...)
#endif

#if (LOG_LEVEL < 3)
#define LOGI(...)				customLog(DebugLevel::INFO, __VA_ARGS__)
#else
#define LOGI(...)				(void)(...)
#endif

#if (LOG_LEVEL < 4)
#define LOGW(...)				customLog(DebugLevel::WARNING, __VA_ARGS__)
#else
#define LOGW(...)				(void)(...)
#endif

#if (LOG_LEVEL < 5)
#define LOGE(...)				customLog(DebugLevel::ERROR, __VA_ARGS__)
#else
#define LOGE(...)				(void)(...)
#endif

#endif /* CUSTOM_LOG_H_ */
