/* PCA9554.h
 *
 * Created on: May 13, 2022
 * Author: Martin Danek, martin@embedblog.eu
 */

#ifndef PCA9554_H_
#define PCA9554_H_

#include "hal_inc.h"

#ifndef PCA9554_I2C_TIMEOUT
#define PCA9554_I2C_TIMEOUT		25
#endif

class PCA9554
{
public:
	enum class Reg: uint8_t
	{
		INPUT_DATA = 0x00,			//R, contains input data
		OUTPUT_DATA = 0x01,			//R-W, sets pin output (0 - low, 1 - high)
		POLARITY_INVERSION = 0x02,	//R-W, inverts polarity of input pins (0 - no inversion, 1 - inverted)
		CONFIGURATION = 0x03		//R-W, configures pins as inputs/outputs (0 - output, 1 - input)
	};

	PCA9554(I2C_HandleTypeDef* i2c, uint8_t address = DEFAULT_I2C_ADDRESS);
	PCA9554(I2C_HandleTypeDef* i2c, bool A0, bool A1, bool A2);

	bool init();

	uint8_t readPort(void) const;
	bool readPin(uint8_t pinNumber) const;

	//TODO: output operations

	constexpr static uint8_t DEFAULT_I2C_ADDRESS = 0b00100000;
	constexpr static uint8_t I2C_ADDRESS_A_VERSION = 0b00111000;

private:
	I2C_HandleTypeDef* const m_i2c;
	const uint8_t m_i2c_address;		//stored in 8 bit format, ie. already left-shifted by 1 compared to DEFAULT_I2C_ADDRESS
};



#endif /* PCA9554_H_ */
