#ifndef TMP75_H_
#define TMP75_H_

#include "hal_inc.h"

#ifndef TMP75_I2C_TIMEOUT
#define TMP75_I2C_TIMEOUT		25
#endif

class TMP75
{
public:
	enum class Reg: uint8_t
	{
		TEMPERATURE = 0x00,
		CONFIGURATION = 0x01,
		LOW_LIMIT = 0x02,
		HIGH_LIMIT = 0x03
	};

	enum class Resolution: uint8_t
	{
		BITS_9 = 0b00,		//0.5 oC per LSb
		BITS_10 = 0b01,		//0.25 oC per LSb
		BITS_11 = 0b10,		//0.125 oC per LSb
		BITS_12 = 0b11		//0.0625 oC per LSb
	};

	TMP75(I2C_HandleTypeDef* i2c, uint8_t address = DEFAULT_I2C_ADDRESS);
	TMP75(I2C_HandleTypeDef* i2c, bool A0, bool A1, bool A2);

	bool init(Resolution resolution = Resolution::BITS_12);

	uint16_t readTemperatureRaw(void) const;
	float readTemperature(void);

	void setResolution(Resolution resolution);

	void shutdown(bool isEnabled);
	void singleConversion();

	static constexpr float lsbToDegrees[] = {0.5f, 0.25f, 0.125f, 0.0625f};
	static constexpr uint8_t DEFAULT_I2C_ADDRESS = 0b01001000;

private:
	void setConfigurationRegister(bool startSingleConversion = false) const;

	I2C_HandleTypeDef* const m_i2c;
	const uint8_t m_i2c_address;

	Resolution m_resolution;

	bool m_isShutDown;
};



#endif /* TMP75_H_ */
